import React from 'react'
import PropTypes from 'prop-types'
import { useMeasure } from 'react-use'
import Graph from './graph.jsx'
import Tooltip from './tooltip.jsx'
import Zoomable from './zoomable.jsx'
import Draggable from './draggable.jsx'
import Selectable from './selectable.jsx'
import Nodes from './nodes-styled.jsx'
import Links from './links.jsx'
import * as d3 from 'd3-selection'


const FoundationGraph = ({
    render,
    definitions = [],
    results,
    links
}) => {
    const [ref, {width, height}] = useMeasure()

    return (
        <div ref={ref} style={{ flex: '1 1 auto', height: '100%' }}>
            <Graph
                width={width}
                height={height}
                nodes={results.map(obj => ({ ...obj }))}
                links={links.map(obj => ({ ...obj }))}
            >
                <Tooltip
                    component={props => {
                        const { data: resource } = props
                        const definition = definitions.find(obj => obj.type === resource.type) || {}
                        const Card = definition.card

                        return !Card ? null : (
                            <Card
                                resource={resource}
                                width={243}
                                height={160}
                                style={{
                                    marginLeft: -121.5,
                                    marginTop: -180,
                                    paddingTop: 10
                                }}
                            />
                        )
                    }}
                >
                    {({ show, hide }) => (
                        <Draggable
                            onDragStart={event => {
                                const node = d3.select(event.sourceEvent.target)

                                node.style('cursor', 'grabbing')
                                node.transition().style('fill', '#444')
                                hide()
                            }}
                            onDragEnd={event => {
                                const node = d3.select(event.sourceEvent.target)

                                node.style('cursor', 'grab')
                                node.transition().style('fill', '#fff')
                            }}
                        >
                            <Selectable>
                                <Zoomable scale={results.length > 20 ? 0.5 : 1} />
                                <Links />
                                <Nodes
                                    element="g"
                                    attributes={{}}
                                    onMouseOut={hide}
                                    onMouseOver={show}
                                    onRender={(d, i, groups) => {
                                        const definition = definitions.find(obj => obj.type === d.type) || {}
                                        const Icon = definition.icon || (() => null)

                                        render(
                                            <React.Fragment>
                                                <circle
                                                    r={20}
                                                    fill={definition.color}
                                                />
                                                <Icon
                                                    x={-12}
                                                    y={-12}
                                                    size={24}
                                                    shadowLength={0}
                                                    style={{ pointerEvents: 'none' }}
                                                />
                                            </React.Fragment>,
                                            groups[i]
                                        )
                                    }}
                                    onTick={group => {
                                        group.attr('transform', d => (
                                            `translate(${d.x}, ${d.y})`
                                        ))
                                    }}
                                />
                            </Selectable>
                        </Draggable>
                    )}
                </Tooltip>
            </Graph>
        </div>
    )
}

FoundationGraph.propTypes = {
    render: PropTypes.func.isRequired,
    definitions: PropTypes.array,
    results: PropTypes.array,
    links: PropTypes.array
}

export default FoundationGraph