/**
 * Compare two collections to find links (matching fields). This is useful for
 * prototyping network graphs as you can pass the same list of nodes for both
 * `a` and `b` (i.e. `generateLinks(nodes, nodes)`) to get your link data.
 * 
 * @param  {array}  a       - A list of objects
 * @param  {array}  b       - A list of objects
 * @param  {object} options - Settings used to customize comparisons
 * @return {array}          - A list of link objects representing all matches
 */
export const generateLinks = (a, b, options = {}) => {
	const done = {}
	const output = []
	const compare = options.objectComparator || objectComparator

	for (let i = 0; i < a.length; i++) {
		const first = a[i]
		const firstId = first.id || i

		for (let j = 0; j < b.length; j++) {
			const second = b[j]
			const secondId = second.id || j
			const key = `${firstId}-${secondId}`
			const reversed = `${secondId}-${firstId}`

			if (firstId !== secondId && !done[key] && !done[reversed]) {
				const match = compare(first, second, options)

				done[key] = true
				if (match) output.push({
					source: firstId,
					target: secondId,
					match
				})
			}
		}
	}

	return output
}

/**
 * Compare two objects to find all matched fields.
 * 
 * @param  {object} a       - Any valid object
 * @param  {object} b       - Any valid object
 * @param  {object} options - Settings used to customize comparisons
 * @return {object}         - Listing of all matched keys and values
 */
const objectComparator = (a, b, options = {}) => {
	const { ignored = [] } = options
	const match = {}

	for (let field in a) {
		if (
			a[field] === null ||
			a[field] === undefined ||
			ignored.includes(field)
		) {
			continue
		}

		if (a[field] === b[field]) {
			match[field] = a[field]
			continue
		}

		if (Array.isArray(a[field])) {
			const result = arrayComparator(a[field], b[field], options)
			if (result) match[field] = result
			continue
		}

		if (typeof a[field] === 'object') {
			const result = objectComparator(a[field], b[field], options)
			if (result) match[field] = result
			continue
		}
	}

	return Object.keys(match).length ? match : null
}

/**
 * Compare two arrays to find all matches within their contents.
 * 
 * @param  {array} a - Any valid array
 * @param  {array} b - Any valid array
 * @return {array}   - A list of all matches
 */
const arrayComparator = (a, b, options) => {
	const allPrimitives = a.every(item => /string|boolean|number/.test(typeof item))
	const allArrays = a.every(item => Array.isArray(item))
	const allObjects = !allArrays && a.every(item => typeof item === 'object')

	if (!a.length || !b.length) {
		return null
	}

	if (allPrimitives) {
		const intersection = a.filter(item => b.includes(item))
		return intersection.length ? intersection : null
	}

	if (allObjects) {
		const links = generateLinks(a, b, options)
		return links.length ? links.map(({ match }) => match) : null
	}

	if (allArrays) {
		console.warn('Multi-dimensional arrays are not supported.')
		console.warn(a, b)
		return
	}
	
	console.warn('Mixed arrays are not supported.')
	return null
}