import * as d3 from 'd3-selection'

/**
 * Update a node's selected state based on the current brush area.
 * 
 * @param  {object}  d      - Data backing the node
 * @param  {array}   bounds - Extent of brushed area
 * @return {boolean}        - New selected state of node
 */
export const updateSelectedState = (d, bounds) => {
	const x = bounds[0][0] <= d.x && d.x < bounds[1][0]
	const y = bounds[0][1] <= d.y && d.y < bounds[1][1]
	const selected = x && y

	if (selected && d.previouslySelected) return d.selected = false
	
	return d.selected = selected || d.previouslySelected
}

/**
 * Toggle a node's selected state.
 * 
 * @param {object} d     - Data backing the node
 * @param {number} i     - Index of the node
 * @param {array}  nodes - A list of all node elements
 */
export const selectNode = (d, i, nodes) => {
	// @todo deconflict this with `deselectNodes` to remove need for timeout
	setTimeout(() => {
		d.selected = !d.selected
		d3.select(nodes[i])
			.attr('stroke', d => d.selected ? '#333' : null)
			.attr('stroke-width', d => d.selected ? 2 : null)
	})
}

/**
 * Deselect all nodes unless the shift key is being held.
 * 
 * @param {object} node - A d3 selection of all nodes
 */
export const deselectNodes = node => {
	if (!d3.event.shiftKey) {
		node
			.each(d => d.selected = false)
			.attr('stroke', null)
			.attr('stroke-width', null)
	}
}

/**
 * Callback fired whenever the brushed selection changes.
 * 
 * @param {object} node - A d3 selection of all nodes
 */
export const selectNodes = node => {
    const bounds = d3.event.selection
	
	if (bounds) {
		node.each(d => updateSelectedState(d, bounds))
			.attr('stroke', d => d.selected ? '#333' : null)
			.attr('stroke-width', d => d.selected ? 2 : null)
	}
}

/**
 * Save the previous selected state to differentiate between it and temporary
 * selection changes.
 * 
 * @param {object} node - A d3 selection of all nodes
 */
export const savePreviousSelection = node => {
	node.each(d => {
		d.previouslySelected = d.selected
	})
}